'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _requestPromise = require('request-promise');

var _requestPromise2 = _interopRequireDefault(_requestPromise);

var _semver = require('semver');

var _semver2 = _interopRequireDefault(_semver);

var _fsJetpack = require('fs-jetpack');

var _fsJetpack2 = _interopRequireDefault(_fsJetpack);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _jszip = require('jszip');

var _jszip2 = _interopRequireDefault(_jszip);

var _request = require('request');

var _request2 = _interopRequireDefault(_request);

var _requestProgress = require('request-progress');

var _requestProgress2 = _interopRequireDefault(_requestProgress);

var _cheerio = require('cheerio');

var _cheerio2 = _interopRequireDefault(_cheerio);

var _progressPromise = require('progress-promise');

var _progressPromise2 = _interopRequireDefault(_progressPromise);

var _v = require('uuid/v4');

var _v2 = _interopRequireDefault(_v);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

//Test

var FactorioAPI = function () {
    function FactorioAPI() {
        _classCallCheck(this, FactorioAPI);
    }

    _createClass(FactorioAPI, null, [{
        key: 'init',

        /**
         * This function initializes the api.
         * @param {boolean} [allowMultipleVersions=false]
         * @param {string} [modPath=''] path to the mods folder
         * @param {string} [savePath=''] path to the saves folder
         * @param {string} [gameVersion='0.0.0'] the version of the game
         */
        value: function init() {
            var allowMultipleVersions = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
            var modPath = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
            var savePath = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';
            var downloadPath = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : '';
            var gameVersion = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : "0.0.0";

            this.token = null;
            this.username = null;
            this.modPath = modPath;
            this.savePath = savePath;
            this.downloadPath = downloadPath;
            this.gameVersion = gameVersion;
            this.allowMultipleVersions = allowMultipleVersions;
            this.authenticated = false;
            this.sessionId = '';
            this.cookieJar = _request2.default.jar();
            this.cacheLifespan = 900000;
        }

        /**
         * This function sets the game version.
         * @param {string} gameVersion the version of the game
         */

    }, {
        key: 'setGameVersion',
        value: function setGameVersion(gameVersion) {
            this.gameVersion = gameVersion;
        }

        /**
         * This function sets the mod path.
         * @param {string} modPath path to the mods folder
         */

    }, {
        key: 'setModPath',
        value: function setModPath(modPath) {
            this.modPath = modPath;
        }

        /**
         * This function sets the save path.
         * @param {string} savePath path to the saves folder
         */

    }, {
        key: 'setSavePath',
        value: function setSavePath(savePath) {
            this.savePath = savePath;
        }

        /**
         * This function sets the lifespan of the cache
         * @param {number} lifespan the lifespan in milliseconds
         */

    }, {
        key: 'setCacheLifespan',
        value: function setCacheLifespan(lifespan) {
            this.cacheLifespan = lifespan;
        }

        /**
         * This function gets the lifespan of the cache
         * @returns {number} the lifespan of the cache (default: 900000 ms or 15 minutes)
         */

    }, {
        key: 'getCacheLifespan',
        value: function getCacheLifespan(lifespan) {
            return this.cacheLifespan;
        }

        /**
         * This function gets the save path.
         * @returns {string} path to the saves folder
         */

    }, {
        key: 'getSavePath',
        value: function getSavePath() {
            return this.savePath;
        }

        /**
         * This function gets the mod path.
         * @returns {string} path to the mods folder
         */

    }, {
        key: 'getModPath',
        value: function getModPath() {
            return this.modPath;
        }

        /**
         * This function gets the game version.
         * @returns {string} the version of the game
         */

    }, {
        key: 'getGameVersion',
        value: function getGameVersion() {
            return this.gameVersion;
        }

        /**
         * This function returns true if the api is authenticated
         * @returns {boolean} authenticated
         */

    }, {
        key: 'isAuthenticated',
        value: function isAuthenticated() {
            return this.authenticated;
        }

        /**
         * This function authenticates with a username and password or token, it also downloads and chaches the mod list
         * @param {Object} props The properties for authentication
         * @param {string} props.username The username
         * @param {string} props.password Not recommended, use token instead, but password is required for downloading the Factorio client, unless you manually provide a session id
         * @param {string} props.token The player token, can be found in player-data.json
         * @param {string} props.sessionId Only if you want to download the Factorio client, but you are to paranoid to type your password. You can get this session id by logging in with your browser on factorio.com. You will be able to find a session id in the cookies, by using the devtools of your browser.
         * @returns {Promise} returns a promise
         */

    }, {
        key: 'authenticate',
        value: function authenticate(props) {
            var _this = this;

            return new Promise(function (resolve, reject) {
                if (props.username && props.token) {
                    _this.token = props.token;
                    _this.username = props.username;
                    _this.authenticated = true;
                    resolve();
                } else if (props.username && props.password) {
                    _this.authenticateAPI(props.username, props.password).then(function (body) {
                        _this.username = props.username;
                        _this.token = body[0];
                        _this.authenticated = true;
                        return _this.getCSRFToken();
                    }).then(function (token) {
                        return _this.authenticateWeb(props.username, props.password, token);
                    }).then(function (res) {
                        _this.sessionId = _this.cookieJar.getCookieString('https://www.factorio.com/');
                        resolve();
                    }).catch(function (err) {
                        reject(err);
                    });
                } else if (props.sessionId) {
                    _this.sessionId = props.sessionId;
                    resolve();
                } else {
                    reject("Error: Insufficient information");
                }
            });
        }
    }, {
        key: 'authenticateAPI',
        value: function authenticateAPI(username, password) {
            var apiAuthOptions = {
                method: 'POST',
                uri: 'https://auth.factorio.com/api-login',
                qs: {
                    username: username,
                    password: password,
                    require_ownership: true
                },
                json: true
            };

            return (0, _requestPromise2.default)(apiAuthOptions);
        }
    }, {
        key: 'getCSRFToken',
        value: function getCSRFToken() {
            var getCSRFOptions = {
                method: 'GET',
                uri: 'https://www.factorio.com/login',
                resolveWithFullResponse: true,
                jar: this.cookieJar,
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded'
                }
            };

            return (0, _requestPromise2.default)(getCSRFOptions).then(function (res) {
                var $ = _cheerio2.default.load(res.body);
                return $('input[name=csrf_token]').val();
            });
        }
    }, {
        key: 'authenticateWeb',
        value: function authenticateWeb(username, password, token) {
            var authOptions = {
                method: 'POST',
                url: 'https://www.factorio.com/login',
                form: {
                    csrf_token: token,
                    username_or_email: username,
                    password: password
                },
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded'
                },
                jar: this.cookieJar,
                resolveWithFullResponse: true,
                followAllRedirects: true
            };
            return (0, _requestPromise2.default)(authOptions);
        }

        /** 
         * This function reloads the local cache file with a list of all the available mods
         */

    }, {
        key: 'reloadCache',
        value: function reloadCache() {
            var options = {
                method: 'GET',
                uri: 'https://mods.factorio.com/api/mods',
                qs: {
                    page_size: "max"
                },
                json: true
            };

            return (0, _requestPromise2.default)(options).then(function (body) {
                var cache = { date: Date.now(), body: body };
                return _fsJetpack2.default.writeAsync('mod_cache.json', JSON.stringify(cache));
            });
        }

        /**
         * This function ensures that the mod cache will be valid and up-to-date, and returns the cached mod list
         * @returns {Promise.<Object>} returns the mod list when resolved
         */

    }, {
        key: 'getCache',
        value: function getCache() {
            var _this2 = this;

            return _fsJetpack2.default.existsAsync('mod_cache.json').then(function (res) {
                if (res) {
                    return Promise.resolve();
                } else {
                    return _this2.reloadCache();
                }
            }).then(function () {
                return _this2.readCache();
            }).then(function (cache) {
                if (!cache.date || Date.now() - cache.date > _this2.getCacheLifespan() || Date.now() < cache.date) {
                    return _this2.reloadCache().then(function () {
                        return _this2.readCache();
                    });
                } else {
                    return Promise.resolve(cache);
                }
            });
        }

        /**
         * This function reads the cache file
         * @return {Promise.<Object} the contents of the cache file
         */

    }, {
        key: 'readCache',
        value: function readCache() {
            return _fsJetpack2.default.readAsync('mod_cache.json', 'json');
        }

        /**
         * This function gets the full information of a mod
         * @param {string} name the name of the mod
         * @returns {Promise.<Object>} returns the information about the mod if resolved
         * @see {@link https://wiki.factorio.com/Mod_Portal_API#Result_Entry|Factorio Wiki}
         */

    }, {
        key: 'getMod',
        value: function getMod(name) {
            return new Promise(function (resolve, reject) {
                var options = {
                    method: 'GET',
                    uri: 'https://mods.factorio.com/api/mods/' + name,
                    json: true
                };

                (0, _requestPromise2.default)(options).then(function (body) {
                    resolve(body);
                }).catch(function (err) {
                    reject(err);
                });
            });
        }

        /**
         * This function gets the full information of multiple mods
         * @param {Object[]} mods the mods
         * @param {string} mods[].name the name of the mod
         * @returns {Promise.<Object[]>} returns array of mods with the information about each mod if resolved
         * @see {@link https://wiki.factorio.com/Mod_Portal_API#Result_Entry|Factorio Wiki}
         */

    }, {
        key: 'getMods',
        value: function getMods(mods) {
            var promises = mods.map(function (mod) {
                return getMod(mod.name);
            });
            return Promise.all(promises);
        }

        /**
         * This function searches for mods on the Mod Portal
         * @param {string} query A search query
         * @returns {Promise.<Object>} returns a filtered array with results, sorted from most downloads to least downloads
         */

    }, {
        key: 'searchMods',
        value: function searchMods(query) {
            var _this3 = this;

            return new Promise(function (resolve, reject) {
                _this3.getCache().then(function (cache) {
                    var body = cache.body;
                    var valid = body.results.filter(function (result) {
                        return result.name && result.title;
                    });
                    var results = valid.filter(function (result) {
                        return result.name.toUpperCase().includes(query.toUpperCase());
                    });
                    results = results.sort(function (a, b) {
                        return b.downloads_count - a.downloads_count;
                    });
                    resolve(results);
                }).catch(function (err) {
                    reject(err);
                });
            });
        }

        /**
         * This function can download the game
         * @param {Object} props properties of the file you want to download 
         * @param {string} props.version the version of the game, use 'latest' to get the latest experimental version
         * @param {string} props.build the release build, choose between 'alpha', 'demo' and 'headless'. 'alpha' is the standard full-featured build
         * @param {string} props.distro your OS and architecture, choose between 'win64', 'win64-manual', 'osx', 'linux64'
         * @returns {ProgressPromise} returns a ProgressPromise, use .progress(percentage => {}) to get the progress and .then(() => {}) to see when the download is completed 
         */

    }, {
        key: 'downloadGame',
        value: function downloadGame(props) {
            var _this4 = this;

            return new _progressPromise2.default(function (resolve, reject, prog) {
                var filename = 'file';
                var tempname = (0, _v2.default)();

                (0, _requestProgress2.default)((0, _request2.default)({
                    url: 'https://factorio.com/get-download/' + props.version + '/' + props.build + '/' + props.distro,
                    headers: {
                        'Cookie': _this4.sessionId
                    },
                    jar: _this4.cookieJar
                })).on('progress', function (state) {
                    prog(state.percent);
                }).on('error', function (err) {
                    reject(err);
                }).on('response', function (res) {
                    filename = res.headers['content-disposition'].split("filename=")[1];
                }).on('end', function () {
                    _fsJetpack2.default.rename(_path2.default.join(_this4.downloadPath, tempname + '.temp'), _path2.default.join(_this4.downloadPath, filename));
                    resolve();
                }).pipe(_fsJetpack2.default.createWriteStream(_path2.default.join(_this4.downloadPath, tempname + '.temp')));
            });
        }

        /**
         * Get the version number of the latest version of the client
         * @param {string} buildType build type, 'stable' or 'experimental' 
         * @returns {Promise.<string>} returns the version number if resolved
         */

    }, {
        key: 'getLatestGameVersion',
        value: function getLatestGameVersion(buildType) {
            var url = '';
            if (buildType == 'experimental') {
                url = 'https://www.factorio.com/download/experimental';
            } else {
                url = 'https://www.factorio.com/download';
            }

            var options = {
                method: 'GET',
                url: url,
                headers: {
                    'Cookie': this.sessionId
                }
            };

            return (0, _requestPromise2.default)(options).then(function (body) {
                var $ = _cheerio2.default.load(body);
                return $('h3').first().text().split('(')[0].trim();
            });
        }

        /**
         * This function downloads the latest version of a mod, compatible with the given game version, if there's a more recent version available
         * @param {Object} mod the mod you want to update
         * @param {string} mod.name the name of the mod
         * @param {string} mod.version the version you have installed
         * @param {string} [factorioVersion=this.gameVersion] the version of the game
         * @returns {Promise.<Object>} returns the information about the mod if resolved
         * @see {@link https://wiki.factorio.com/Mod_Portal_API#Result_Entry|Factorio Wiki}
         */

    }, {
        key: 'updateMod',
        value: function updateMod(mod) {
            var _this5 = this;

            var factorioVersion = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.gameVersion;

            return new Promise(function (resolve, reject) {
                _this5.checkUpdate(mod, factorioVersion).then(function (result) {
                    if (result.hasUpdate) {
                        _this5.downloadMod({ name: mod.name, version: result.version }).then(function () {
                            resolve(result);
                        }).catch(function (err) {
                            reject(err);
                        });
                    } else {
                        resolve(result);
                    }
                }).catch(function (err) {
                    reject(err);
                });
            });
        }

        /**
         * This function updates all given mods and resolves if all mods are downloaded
         * @param {Object[]} mods the mods you want to update
         * @param {string} mod[].name the name of the mod
         * @param {string} mod[].version the version you have installed
         * @param {string} [factorioVersion=this.gameVersion] the version of the game
         * @returns {Promise.<Object>} returns the information about each mod if resolved
         * @see {@link https://wiki.factorio.com/Mod_Portal_API#Result_Entry|Factorio Wiki}
         */

    }, {
        key: 'updateMods',
        value: function updateMods(mods) {
            var _this6 = this;

            var factorioVersion = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.gameVersion;

            var promises = mods.map(function (mod) {
                return _this6.updateMod(mod, factorioVersion);
            });
            return Promise.all(promises);
        }

        /**
         * This function checks for updates for a mod, compatible with the given game version
         * @param {Object} mod the mod you want to update
         * @param {string} mod.name the name of the mod
         * @param {string} mod.version the version you have installed
         * @param {string} [factorioVersion=this.gameVersion] the version of the game
         * @returns {Promise.<Object>} returns {onlineMod, hasUpdate: true, version: release.version} if an update is available and returns {onlineMod, hasUpdate: false} when the mod is up-to-date
         */

    }, {
        key: 'checkUpdate',
        value: function checkUpdate(mod) {
            var _this7 = this;

            var factorioVersion = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.gameVersion;

            return new Promise(function (resolve, reject) {
                _this7.getMod(mod.name).then(function (onlineMod) {
                    if (factorioVersion.split('.').length < 3) {
                        factorioVersion += '.0';
                    }
                    onlineMod.releases.forEach(function (release) {
                        if (release.info_json.factorio_version.split('.').length < 3) {
                            release.info_json.factorio_version += '.0';
                        }
                        if (_semver2.default.gt(release.version, mod.version) && (_semver2.default.minor(release.info_json.factorio_version) == _semver2.default.minor(factorioVersion) || factorioVersion == "0.0.0")) {
                            resolve({ onlineMod: onlineMod, hasUpdate: true, version: release.version });
                        }
                    });

                    resolve({ onlineMod: onlineMod, hasUpdate: false });
                }).catch(function (err) {
                    reject(err);
                });
            });
        }

        /**
         * This function checks for updates for all given mods
         * @param {Object[]} mods the mods you want to update
         * @param {string} mods[].name the name of the mod
         * @param {string} mods[].version the version you have installed
         * @param {string} [factorioVersion=this.gameVersion] the version of the game
         * @returns {Promise.<Object[]>} returns an array of all objects returned by checkUpdate
         */

    }, {
        key: 'checkUpdates',
        value: function checkUpdates(mods) {
            var _this8 = this;

            var factorioVersion = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.gameVersion;

            var promises = mods.map(function (mod) {
                return _this8.checkUpdate(mod, factorioVersion);
            });
            return Promise.all(promises);
        }

        /**
         * This function downloads a mod
         * @param {Object} mod the mod you want to download
         * @param {string} mod.name the name of the mod
         * @param {string} mod.version the version you want to download (optional, default is latest)
         * @param {string} [factorioVersion=this.gameVersion] the version of the game
         * @returns {Promise.<Object>} returns the online mod information if resolved
         * @see {@link https://wiki.factorio.com/Mod_Portal_API#Result_Entry|Factorio Wiki}
         */

    }, {
        key: 'downloadMod',
        value: function downloadMod(mod) {
            var _this9 = this;

            return new Promise(function (resolve, reject) {
                _this9.getMod(mod.name).then(function (onlineMod) {
                    var release = void 0;

                    if (mod.version) {
                        release = onlineMod.releases.find(function (x) {
                            return x.version == mod.version;
                        });
                    } else {
                        release = onlineMod.releases[onlineMod.releases.length - 1];
                    }

                    _this9.downloadModFromUrl(release.file_name, release.download_url).then(function () {
                        resolve(onlineMod);
                    }).catch(function (err) {
                        reject(err);
                    });
                }).catch(function (err) {
                    reject(err);
                });
            });
        }

        /**
         * This function downloads mods
         * @param {Object} mods the mods you want to download
         * @param {string} mods[].name the name of the mod
         * @param {string} mods[].version the version you want to download (optional, default is latest)
         * @param {string} [factorioVersion=this.gameVersion] the version of the game
         * @returns {Promise.<Object[]>} returns array of online mods if resolved
         * @see {@link https://wiki.factorio.com/Mod_Portal_API#Result_Entry|Factorio Wiki}
         */

    }, {
        key: 'downloadMods',
        value: function downloadMods(mods) {
            var _this10 = this;

            var promises = mods.map(function (mod) {
                return _this10.downloadMod(mod);
            });
            return Promise.all(promises);
        }

        /**
         * This function downloads a mod from a url
         * @param {string} fileName the name of the file
         * @param {string} url the download_url of a release (example: /api/downloads/data/mods/id/name_version.zip)
         * @returns {ProgressPromise.<string>} returns name of the mod if resolved
         * @see {@link https://wiki.factorio.com/Mod_Portal_API#Releases|Factorio Wiki}
         */

    }, {
        key: 'downloadModFromUrl',
        value: function downloadModFromUrl(fileName, url) {
            var _this11 = this;

            return new Promise(function (resolve, reject) {
                var fullUrl = 'https://mods.factorio.com' + url + ('?username=' + _this11.username + '&token=' + _this11.token);
                var name = fileName.replace(fileName.substr(fileName.lastIndexOf('_')), '');

                (0, _requestProgress2.default)((0, _request2.default)(fullUrl)).on('progress', function (state) {}).on('error', function (err) {
                    reject(err);
                }).on('end', function () {
                    if (!_this11.allowMultipleVersions) {
                        _fsJetpack2.default.findAsync(_this11.modPath, { matching: name + '_*.zip' }).then(function (files) {
                            var promises = [];

                            files.forEach(function (file) {
                                if (_path2.default.basename(file) != fileName) {
                                    promises.push(_fsJetpack2.default.removeAsync(file));
                                }
                            });

                            Promise.all(promises).then(function () {
                                resolve(name);
                            }).catch(function (err) {
                                reject(err);
                            });
                        });
                    } else {
                        resolve(name);
                    }
                }).pipe(_fsJetpack2.default.createWriteStream(_path2.default.join(_this11.modPath, fileName)));
            });
        }

        /**
         * This function removes all mod zip files matching a glob pattern (relative to mod path)
         * @param {Object} mod the mod you want to remove
         * @param {string} mod.name the name of the mod, can be a glob pattern
         * @param {string} mod.version the version you want to remove, can be a glob pattern too (optional, if not specified, all versions will be removed)
         * @returns {Promise} resolves if successful
         */

    }, {
        key: 'removeModsMatching',
        value: function removeModsMatching(mod) {
            var _this12 = this;

            return new Promise(function (resolve, reject) {
                var promises = [];

                if (!mod.version) {
                    mod.version = "*";
                }

                _fsJetpack2.default.findAsync(_this12.modPath, { matching: mod.name + '_' + mod.version + '.zip' }).then(function (files) {
                    files.forEach(function (file) {
                        promises.push(_fsJetpack2.default.removeAsync(file));
                    });
                });

                Promise.all(promises).then(function () {
                    resolve();
                }).catch(function (err) {
                    reject(err);
                });
            });
        }

        /**
         * This function removes all mod zip files from the mods in the array
         * @param {Object[]} mods the mods you want to remove
         * @param {string} mod[].name the name of the mod, can be a glob pattern
         * @param {string} mod[].version the version you want to remove, can be a glob pattern too (optional, if not specified, all versions will be removed)
         * @returns {Promise} resolves if successful
         */

    }, {
        key: 'removeMods',
        value: function removeMods(mods) {
            var _this13 = this;

            return Promise.all(mods.map(function (mod) {
                return _this13.removeModsMatching(mod);
            }));
        }

        /**
         * This function downloads all dependencies for a mod
         * @param {Object} mod the mod
         * @param {string} mod.name the name of the mod
         * @param {boolean} [optionalMods=false] set to true if you want to download all optional mods (I don't know why you would ever want to do that, but you can!)
         * @returns {Promise.<Object>} returns all online mods if resolved
         */

    }, {
        key: 'downloadDependencies',
        value: function downloadDependencies(mod) {
            var _this14 = this;

            var optionalMods = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

            return new Promise(function (resolve, reject) {
                _this14.getDependencies(mod, optionalMods).then(function (dependencies) {
                    return _this14.downloadMods(dependencies);
                }).then(function (mods) {
                    resolve(mods);
                }).catch(function (err) {
                    reject(err);
                });
            });
        }

        /**
         * This function gets all dependencies for a mod
         * @param {Object} mod the mod you want to get info.json from
         * @param {string} mod.name name of the mod
         * @param {boolean} [optionalMods=false] set to true if you want to download all optional mods (I don't know why you would ever want to do that, but you can!)
         * @returns {Promise.<Object[]>} returns array of mods with their name as property ([{name: name_of_the_mod}]) if resolved
         */

    }, {
        key: 'getDependencies',
        value: function getDependencies(mod) {
            var _this15 = this;

            var optionalMods = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

            return new Promise(function (resolve, reject) {
                _this15.readModZipFile(_path2.default.basename(_fsJetpack2.default.find(_this15.modPath, { matching: mod.name + '_*.zip' })[0])).then(function (info_json) {
                    var dependencies = info_json.dependencies;

                    if (!optionalMods) {
                        // Ignore "base" and dependencies that start with "?"
                        dependencies = dependencies.filter(function (x) {
                            return x[0] != "?" && x.substring(0, 4) != "base";
                        });
                    } else {
                        dependencies = dependencies.map(function (x) {
                            if (x[0] == "?") {
                                // Remove "? "
                                return x.substring(2);
                            } else {
                                return x;
                            }
                        });

                        // Ignore "base"
                        dependencies = dependencies.filter(function (x) {
                            return x.substring(0, 4) != "base";
                        });
                    }

                    // Ignore version number, only look at name and create object with name
                    dependencies = dependencies.map(function (x) {
                        return { name: x.split(" ")[0] };
                    });
                    resolve(dependencies);
                }).catch(function (err) {
                    reject(err);
                });
            });
        }

        /**
         * This function gets all available online games
         * @returns {Promise.<Object>} returns result of the request (https://multiplayer.factorio.com/get-games?username=<your_username>&token=<your_token>)
         * @see {@link https://wiki.factorio.com/Matchmaking_API#get-games|Factorio Wiki}
         */

    }, {
        key: 'getGames',
        value: function getGames() {
            var options = {
                method: 'GET',
                uri: 'https://multiplayer.factorio.com/get-games',
                qs: {
                    username: this.username,
                    token: this.token
                },
                json: true
            };

            return (0, _requestPromise2.default)(options);
        }

        /**
         * This function gets details of a game
         * @param {string} gameId the id of an online game (get it from getGames())
         * @returns {Promise.<Object>} returns result of the request (https://multiplayer.factorio.com/get-games?username=<your_username>&token=<your_token>)
         * @see {@link https://wiki.factorio.com/Matchmaking_API#get-game-details|Factorio Wiki}
         */

    }, {
        key: 'getGameDetails',
        value: function getGameDetails(gameId) {
            var options = {
                method: 'GET',
                uri: 'https://multiplayer.factorio.com/get-game-details/' + gameId,
                json: true
            };

            return (0, _requestPromise2.default)(options);
        }

        /**
         * This function gets mods from a save file
         * @param {string} fileName name of the file, with .zip (relative to save path)
         * @param {boolean} [includeFileName=false] if set to false, an array with all mods will be returned (mod is an object with name and version). If set to true, it will return an object with a name property and a mods property resolve({name: name_of_save, mods: mods})
         * @returns {Promise.<Object>}
         */

    }, {
        key: 'getModsFromSaveFile',
        value: function getModsFromSaveFile(fileName) {
            var _this16 = this;

            var includeFileName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

            return new Promise(function (resolve, reject) {
                var file = _path2.default.join(_this16.savePath, fileName);
                _fsJetpack2.default.readAsync(file, 'buffer').then(function (buf) {
                    _jszip2.default.loadAsync(buf).then(function (zip) {
                        return zip.file(/level-init\.dat/)[0].async('nodebuffer');
                    }).then(function (buffer) {
                        var mods = [];
                        var modCount = buffer.readUIntBE(48, 1);
                        for (var i = modCount, pos = 52; i > 0; i--) {
                            var length = buffer.readUIntBE(pos, 1);

                            var modName = buffer.toString('utf-8', pos + 1, pos + length + 1);
                            var vMajor = buffer.readUIntBE(pos + length + 1, 1);
                            var vMinor = buffer.readUIntBE(pos + length + 2, 1);
                            var vPatch = buffer.readUIntBE(pos + length + 3, 1);

                            var fullVersion = 'v' + vMajor + '.' + vMinor + '.' + vPatch;

                            mods.push({ name: modName, version: fullVersion });

                            pos += length + 8;
                        }

                        if (includeFileName) {
                            resolve({ name: fileName.replace('.zip', ''), mods: mods });
                        } else {
                            resolve(mods);
                        }
                    }).catch(function (err) {
                        reject(err);
                    });
                });
            });
        }

        /**
         * This function gets mods from a save file
         * @param {string} saveName name of the save, without .zip (relative to save path)
         * @param {boolean} [includeFileName=false] if set to false, an array with all mods will be returned (mod is an object with name and version). If set to true, it will return an object with a name property and a mods property resolve({name: name_of_save, mods: mods})
         * @returns {Promise.<Object>}
         */

    }, {
        key: 'getModsFromSave',
        value: function getModsFromSave(saveName) {
            var includeFileName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

            var fileName = saveName + '.zip';
            return this.getModsFromSaveFile(fileName, includeFileName);
        }

        /**
         * This function gets mods from all save files
         * @returns {Promise.<Object[]>} see docs for getModsFromSave for more info, includeFileName is always true (otherwise it would be confusing)
         */

    }, {
        key: 'getModsFromSaves',
        value: function getModsFromSaves() {
            var _this17 = this;

            return new Promise(function (resolve, reject) {
                _fsJetpack2.default.findAsync(_this17.savePath, { matching: '*.zip' }).then(function (files) {
                    Promise.all(files.map(function (x) {
                        return _this17.getModsFromSaveFile(_path2.default.basename(x), true);
                    })).then(function (list) {
                        resolve(list);
                    }).catch(function (err) {
                        reject(err);
                    });
                }).catch(function (err) {
                    reject(err);
                });
            });
        }

        /**
         * This function reads info.json from a zip file of a mod
         * @param {string} fileName name of the mod, with .zip (relative to mod path)
         * @returns {Promise.<Object>} returns parsed info.json file if resolved
         */

    }, {
        key: 'readModZipFile',
        value: function readModZipFile(fileName) {
            var _this18 = this;

            return new Promise(function (resolve, reject) {
                var file = _path2.default.join(_this18.modPath, fileName);
                _fsJetpack2.default.readAsync(file, 'buffer').then(function (buffer) {
                    _jszip2.default.loadAsync(buffer).then(function (zip) {
                        return zip.file(/info\.json/)[0].async('text');
                    }).then(function (data) {
                        data = JSON.parse(data);
                        resolve(data);
                    }).catch(function (err) {
                        reject(err);
                    });
                }).catch(function (err) {
                    reject(err);
                });
            });
        }

        /**
         * This function reads info.json from a zip file of a mod
         * @param {Object} mod the mod you want to get info.json from
         * @param {string} mod.name name of the mod
         * @param {string} mod.version version of the mod
         * @returns {Promise.<Object>} returns parsed info.json file if resolved
         */

    }, {
        key: 'readModZip',
        value: function readModZip(mod) {
            var fileName = mod.name + '_' + mod.version + '.zip';
            return this.readModZipFile(fileName);
        }

        /**
         * This function reads info.json from all zip files in mod path
         * @returns {Promise.<Object[]>} returns array of parsed info.json files (sorted alphabetically) if resolved
         */

    }, {
        key: 'readModZips',
        value: function readModZips() {
            var _this19 = this;

            return new Promise(function (resolve, reject) {
                _fsJetpack2.default.findAsync(_this19.modPath, { matching: '*_*.zip' }).then(function (files) {
                    Promise.all(files.map(function (x) {
                        return _this19.readModZipFile(_path2.default.basename(x));
                    })).then(function (list) {
                        list = list.sort(function (a, b) {
                            return a.name.localeCompare(b.name);
                        });
                        resolve(list);
                    }).catch(function (err) {
                        reject(err);
                    });
                }).catch(function (err) {
                    reject(err);
                });
            });
        }

        /**
         * This function reads info.json from all zip files in mod path and applies information from mod-list.json to it (enabled or disabled)
         * @returns {Promise.<Object[]>} returns array of parsed info.json files (sorted alphabetically) with enabled property if resolved
         */

    }, {
        key: 'loadInstalledMods',
        value: function loadInstalledMods() {
            var _this20 = this;

            return new Promise(function (resolve, reject) {
                _this20.readModZips().then(function (list) {
                    if (!_fsJetpack2.default.exists(_path2.default.join(_this20.modPath, 'mod-list.json'))) {
                        _fsJetpack2.default.write(_path2.default.join(_this20.modPath, 'mod-list.json'), { mods: [] });
                    }
                    _fsJetpack2.default.readAsync(_path2.default.join(_this20.modPath, 'mod-list.json'), 'json').then(function (modListJson) {
                        list = list.map(function (mod) {
                            var jsonMod = modListJson.mods.find(function (x) {
                                return x.name == mod.name;
                            });
                            if (jsonMod) {
                                mod.enabled = jsonMod.enabled == 'true';
                            } else {
                                mod.enabled = true;
                            }
                            return mod;
                        });
                        list = list.sort(function (a, b) {
                            return a.name.localeCompare(b.name);
                        });
                        resolve(list);
                    }).catch(function (err) {
                        reject(err);
                    });
                }).catch(function (err) {
                    reject(err);
                });
            });
        }

        /**
         * This function gets the names and versions of the mods in the mods folder
         * @returns {Promise.<Object[]>} returns array of objects with name and version of each mod
         */

    }, {
        key: 'getInstalledMods',
        value: function getInstalledMods() {
            var _this21 = this;

            return new Promise(function (resolve, reject) {
                _fsJetpack2.default.findAsync(_this21.modPath, { matching: '*_*.zip' }).then(function (files) {
                    var nameArr = files.map(function (file) {
                        return {
                            name: file.substring(file.indexOf("/") + 1, file.lastIndexOf("_")),
                            version: file.substring(file.lastIndexOf("_") + 1, file.lastIndexOf(".zip"))
                        };
                    });
                    resolve(nameArr);
                }).catch(function (err) {
                    reject(err);
                });
            });
        }

        /**
         * This function saves the mod list
         * @param {Object[]} mods all the installed mods
         * @param {string} mods[].name name of the mod
         * @param {string} mods[].enabled enabled property of the mod
         * @returns {Promise}
         */

    }, {
        key: 'saveModList',
        value: function saveModList(mods) {
            mods = mods.map(function (mod) {
                return { name: mod.name, enabled: mod.enabled.toString() };
            });
            return _fsJetpack2.default.writeAsync(_path2.default.join(this.modPath, 'mod-list.json'), JSON.stringify({ mods: mods }));
        }
    }]);

    return FactorioAPI;
}();

module.exports = FactorioAPI;